module expat;

/* Copyright (c) 1998, 1999, 2000 Thai Open Source Software Center Ltd
    See the file COPYING for copying permission.
*/

// Converted to D from C header 
// Brian Hammond <d-expat at brianhammond dot com>
// Mon May 03 22:21:57 EDT 2004 

struct XML_ParserStruct;
alias XML_ParserStruct* XML_Parser;

alias char XML_Char;
alias char XML_LChar;

alias ubyte XML_Bool;
XML_Bool XML_TRUE = 1;
XML_Bool XML_FALSE = 0;

enum 
{
    XML_STATUS_ERROR = 0,
    XML_STATUS_OK = 1
}

alias int XML_Status;

enum 
{
    XML_ERROR_NONE,
    XML_ERROR_NO_MEMORY,
    XML_ERROR_SYNTAX,
    XML_ERROR_NO_ELEMENTS,
    XML_ERROR_INVALID_TOKEN,
    XML_ERROR_UNCLOSED_TOKEN,
    XML_ERROR_PARTIAL_CHAR,
    XML_ERROR_TAG_MISMATCH,
    XML_ERROR_DUPLICATE_ATTRIBUTE,
    XML_ERROR_JUNK_AFTER_DOC_ELEMENT,
    XML_ERROR_PARAM_ENTITY_REF,
    XML_ERROR_UNDEFINED_ENTITY,
    XML_ERROR_RECURSIVE_ENTITY_REF,
    XML_ERROR_ASYNC_ENTITY,
    XML_ERROR_BAD_CHAR_REF,
    XML_ERROR_BINARY_ENTITY_REF,
    XML_ERROR_ATTRIBUTE_EXTERNAL_ENTITY_REF,
    XML_ERROR_MISPLACED_XML_PI,
    XML_ERROR_UNKNOWN_ENCODING,
    XML_ERROR_INCORRECT_ENCODING,
    XML_ERROR_UNCLOSED_CDATA_SECTION,
    XML_ERROR_EXTERNAL_ENTITY_HANDLING,
    XML_ERROR_NOT_STANDALONE,
    XML_ERROR_UNEXPECTED_STATE,
    XML_ERROR_ENTITY_DECLARED_IN_PE,
    XML_ERROR_FEATURE_REQUIRES_XML_DTD,
    XML_ERROR_CANT_CHANGE_FEATURE_ONCE_PARSING,
    XML_ERROR_UNBOUND_PREFIX
}

alias int XML_Error;

enum 
{
    XML_CTYPE_EMPTY = 1,
    XML_CTYPE_ANY,
    XML_CTYPE_MIXED,
    XML_CTYPE_NAME,
    XML_CTYPE_CHOICE,
    XML_CTYPE_SEQ
}

alias int XML_Content_Type;

enum 
{
    XML_CQUANT_NONE,
    XML_CQUANT_OPT,
    XML_CQUANT_REP,
    XML_CQUANT_PLUS
}

alias int XML_Content_Quant;

alias XML_cp XML_Content;

struct XML_cp 
{
    XML_Content_Type type;
    XML_Content_Quant quant;
    XML_Char* name;
    uint numchildren;
    XML_Content *children;
}

alias void (*XML_ElementDeclHandler) (void* userData, XML_Char* name, XML_Content *model);
extern(C) void XML_SetElementDeclHandler(XML_Parser parser, XML_ElementDeclHandler eldecl);

alias void (*XML_AttlistDeclHandler) (void* userData, XML_Char* elname, XML_Char* attname, XML_Char* att_type, XML_Char* dflt, int isrequired);
extern(C) void XML_SetAttlistDeclHandler(XML_Parser parser, XML_AttlistDeclHandler attdecl); 

alias void (*XML_XmlDeclHandler) (void* userData, XML_Char* ver, XML_Char* encoding, int standalone);
extern(C) void XML_SetXmlDeclHandler(XML_Parser parser, XML_XmlDeclHandler xmldecl);

struct XML_Memory_Handling_Suite 
{
    void* (*malloc_fcn)(size_t size);
    void* (*realloc_fcn)(void* ptr, size_t size);
    void (*free_fcn)(void* ptr);
}

extern(C) XML_Parser XML_ParserCreate(XML_Char* encoding);
extern(C) XML_Parser XML_ParserCreateNS(XML_Char* encoding, XML_Char* namespaceSeparator);
extern(C) XML_Parser XML_ParserCreate_MM(XML_Char* encoding, XML_Memory_Handling_Suite *memsuite, XML_Char* namespaceSeparator);
extern(C) XML_Bool XML_ParserReset(XML_Parser parser, XML_Char* encoding);

alias void (*XML_StartElementHandler) (void* userData, XML_Char* name, XML_Char** atts);
alias void (*XML_EndElementHandler) (void* userData, XML_Char* name);
alias void (*XML_CharacterDataHandler) (void* userData, XML_Char* s, int len);
alias void (*XML_ProcessingInstructionHandler) (void* userData, XML_Char* target, XML_Char* data);
alias void (*XML_CommentHandler) (void* userData, XML_Char* data);
alias void (*XML_StartCdataSectionHandler) (void* userData);
alias void (*XML_EndCdataSectionHandler) (void* userData);
alias void (*XML_DefaultHandler) (void* userData, XML_Char* s, int len);
alias void (*XML_StartDoctypeDeclHandler) (void* userData, XML_Char* doctypeName, XML_Char* sysid, XML_Char* pubid, int has_internal_subset);
alias void (*XML_EndDoctypeDeclHandler)(void* userData);
alias void (*XML_EntityDeclHandler) (void* userData, XML_Char* entityName, int is_parameter_entity, XML_Char* value, int value_length, XML_Char* base, XML_Char* systemId, XML_Char* publicId, XML_Char* notationName);

extern(C) void XML_SetEntityDeclHandler(XML_Parser parser, XML_EntityDeclHandler handler);

alias void (*XML_UnparsedEntityDeclHandler) (void* userData, XML_Char* entityName, XML_Char* base, XML_Char* systemId, XML_Char* publicId, XML_Char* notationName);
alias void (*XML_NotationDeclHandler) (void* userData, XML_Char* notationName, XML_Char* base, XML_Char* systemId, XML_Char* publicId);
alias void (*XML_StartNamespaceDeclHandler) (void* userData, XML_Char* prefix, XML_Char* uri);
alias void (*XML_EndNamespaceDeclHandler) (void* userData, XML_Char* prefix);
alias int (*XML_NotStandaloneHandler) (void* userData);
alias int (*XML_ExternalEntityRefHandler) (XML_Parser parser, XML_Char* context, XML_Char* base, XML_Char* systemId, XML_Char* publicId);
alias void (*XML_SkippedEntityHandler) (void* userData, XML_Char* entityName, int is_parameter_entity);

struct XML_Encoding 
{
    int map[256];
    void* data;
    int (*convert)(void* data, char *s);
    void (*release)(void* data);
}

alias int (*XML_UnknownEncodingHandler) (void* encodingHandlerData, XML_Char* name, XML_Encoding *info);

extern(C):

void XML_SetElementHandler(XML_Parser parser, XML_StartElementHandler start, XML_EndElementHandler end);
void XML_SetStartElementHandler(XML_Parser, XML_StartElementHandler);
void XML_SetEndElementHandler(XML_Parser, XML_EndElementHandler);
void XML_SetCharacterDataHandler(XML_Parser parser, XML_CharacterDataHandler handler);
void XML_SetProcessingInstructionHandler(XML_Parser parser, XML_ProcessingInstructionHandler handler);
void XML_SetCommentHandler(XML_Parser parser, XML_CommentHandler handler);
void XML_SetCdataSectionHandler(XML_Parser parser, XML_StartCdataSectionHandler start, XML_EndCdataSectionHandler end);
void XML_SetStartCdataSectionHandler(XML_Parser parser, XML_StartCdataSectionHandler start);
void XML_SetEndCdataSectionHandler(XML_Parser parser, XML_EndCdataSectionHandler end);
void XML_SetDefaultHandler(XML_Parser parser, XML_DefaultHandler handler);
void XML_SetDefaultHandlerExpand(XML_Parser parser, XML_DefaultHandler handler);
void XML_SetDoctypeDeclHandler(XML_Parser parser, XML_StartDoctypeDeclHandler start, XML_EndDoctypeDeclHandler end);
void XML_SetStartDoctypeDeclHandler(XML_Parser parser, XML_StartDoctypeDeclHandler start);
void XML_SetEndDoctypeDeclHandler(XML_Parser parser, XML_EndDoctypeDeclHandler end);
void XML_SetUnparsedEntityDeclHandler(XML_Parser parser, XML_UnparsedEntityDeclHandler handler);
void XML_SetNotationDeclHandler(XML_Parser parser, XML_NotationDeclHandler handler);
void XML_SetNamespaceDeclHandler(XML_Parser parser, XML_StartNamespaceDeclHandler start, XML_EndNamespaceDeclHandler end);
void XML_SetStartNamespaceDeclHandler(XML_Parser parser, XML_StartNamespaceDeclHandler start);
void XML_SetEndNamespaceDeclHandler(XML_Parser parser, XML_EndNamespaceDeclHandler end);
void XML_SetNotStandaloneHandler(XML_Parser parser, XML_NotStandaloneHandler handler);
void XML_SetExternalEntityRefHandler(XML_Parser parser, XML_ExternalEntityRefHandler handler);
void XML_SetExternalEntityRefHandlerArg(XML_Parser, void* arg);
void XML_SetSkippedEntityHandler(XML_Parser parser, XML_SkippedEntityHandler handler);
void XML_SetUnknownEncodingHandler(XML_Parser parser, XML_UnknownEncodingHandler handler, void* encodingHandlerData);
void XML_DefaultCurrent(XML_Parser parser);
void XML_SetReturnNSTriplet(XML_Parser parser, int do_nst);
void XML_SetUserData(XML_Parser parser, void* userData);

// TODO unsupported so we don't have to create an import lib
//#define XML_GetUserData(parser) (*(void**)(parser))

XML_Status XML_SetEncoding(XML_Parser parser, XML_Char* encoding);
void XML_UseParserAsHandlerArg(XML_Parser parser);
XML_Error XML_UseForeignDTD(XML_Parser parser, XML_Bool useDTD);
XML_Status XML_SetBase(XML_Parser parser, XML_Char* base);
XML_Char* XML_GetBase(XML_Parser parser);
int XML_GetSpecifiedAttributeCount(XML_Parser parser);
int XML_GetIdAttributeIndex(XML_Parser parser);
XML_Status XML_Parse(XML_Parser parser, char *s, int len, int isFinal);
void* XML_GetBuffer(XML_Parser parser, int len);
XML_Status XML_ParseBuffer(XML_Parser parser, int len, int isFinal);
XML_Parser XML_ExternalEntityParserCreate(XML_Parser parser, XML_Char* context, XML_Char* encoding);

enum 
{
    XML_PARAM_ENTITY_PARSING_NEVER,
    XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE,
    XML_PARAM_ENTITY_PARSING_ALWAYS
}

alias int XML_ParamEntityParsing;

int XML_SetParamEntityParsing(XML_Parser parser, XML_ParamEntityParsing parsing);
XML_Error XML_GetErrorCode(XML_Parser parser);
int XML_GetCurrentLineNumber(XML_Parser parser);
int XML_GetCurrentColumnNumber(XML_Parser parser);
long XML_GetCurrentByteIndex(XML_Parser parser);
int XML_GetCurrentByteCount(XML_Parser parser);
char* XML_GetInputContext(XML_Parser parser, int* offset, int* size);

alias XML_GetCurrentLineNumber XML_GetErrorLineNumber;
alias XML_GetCurrentColumnNumber XML_GetErrorColumnNumber;
alias XML_GetCurrentByteIndex XML_GetErrorByteIndex;

void XML_FreeContentModel(XML_Parser parser, XML_Content *model);
void* XML_MemMalloc(XML_Parser parser, size_t size);
void* XML_MemRealloc(XML_Parser parser, void* ptr, size_t size);
void XML_MemFree(XML_Parser parser, void* ptr);
void XML_ParserFree(XML_Parser parser);

XML_LChar* XML_ErrorString(XML_Error code);
XML_LChar* XML_ExpatVersion();

struct XML_Expat_Version
{
    int major;
    int minor;
    int micro;
} 

XML_Expat_Version XML_ExpatVersionInfo();

enum 
{
    XML_FEATURE_END = 0,
    XML_FEATURE_UNICODE,
    XML_FEATURE_UNICODE_WCHAR_T,
    XML_FEATURE_DTD,
    XML_FEATURE_CONTEXT_BYTES,
    XML_FEATURE_MIN_SIZE,
    XML_FEATURE_SIZEOF_XML_CHAR,
    XML_FEATURE_SIZEOF_XML_LCHAR
}

alias int XML_FeatureEnum;

struct XML_Feature 
{
    XML_FeatureEnum feature;
    XML_LChar *name;
    long value;
}

XML_Feature* XML_GetFeatureList();

const int XML_MAJOR_VERSION = 1;
const int XML_MINOR_VERSION = 95;
const int XML_MICRO_VERSION = 7;

